#!/usr/bin/python

##############################################################################################
# Copyright (c) by George Ruinelli 
# License:     GPL3
# Description:    This application shows the angle in a graph.
#            It is primarly an example applications which shows the 
#        interactions between QML and PySide.
#        Also it shows how a graph can be created in Python and shown in QML
##############################################################################################


import sys
from PySide.QtCore import *
from PySide.QtGui import *
from PySide.QtDeclarative import *

import random
import math
import platform
from PySide.QtGui import QDesktopServices as QDS


import data #own data

#own functions
from functions_general import *
from functions_file import *
from functions_record import *
from functions_alarm import *








    
################################################################################################    

def main():
    global app, config,  root
    app = QApplication(sys.argv)

    #qmlRegisterType(NAME OF CLASS,     QML IMPORT NAME,     VERSION MAJOR,     VERSION MINOR,     COMPONENT NAME IN QML)
    qmlRegisterType(Graph, 'myGraph', 1, 0, 'Graph')

    view = QDeclarativeView()
    view.setSource(QUrl.fromLocalFile('qml/main.qml'))
    root = view.rootObject()


    # instantiate the Python object
    qml_to_python = QML_to_Python_Interface()
    # expose the object to QML
    context = view.rootContext()
    context.setContextProperty("qml_to_python", qml_to_python)
       
    if(platform.machine().startswith('arm')):
      data.DeviceIsN9 = True
   
   
    if(data.DeviceIsN9 == True):
        print "We are on a N9"
        data.configFolder = os.path.join(QDS.storageLocation(QDS.DataLocation), "SleepAnalyser/")
        data.RecordFolder = data.defaultRecordFolder
    else: 
        print "We are NOT on a N9"
        data.configFolder = data.developmentconfigFolder
        data.RecordFolder = data.developmentRecordFolder
    print "data.configFolder:", data.configFolder
    print "data.RecordFolder:", data.RecordFolder
    
    config=LoadConfig()
    
    
#    data.Use_Lucid_Dream_Song = True
#    data.Lucid_Dream_Song="I love rock n roll.mp3"
#    data.WakeUp_Song="Breakfast in america.wav"
    
    root.loadParametersFromPythonLucidDreamMusic(data.Use_Lucid_Dream_Song, data.Lucid_Dream_Song_Delay, data.Lucid_Dream_Song_Volume, data.Lucid_Dream_Song)
    root.loadParametersFromPythonAlarm(data.UseAlarm, data.AlarmHour, data.AlarmMinute, data.AlarmTimeWindow)
    root.loadParametersFromPythonWakeUpMusic(data.Use_WakeUp_Song, data.WakeUp_Song_Volume,  data.WakeUp_Song)
    
    print data.Use_Lucid_Dream_Song, data.Lucid_Dream_Song_Delay, data.Lucid_Dream_Song_Volume, data.Lucid_Dream_Song
    print data.UseAlarm, data.AlarmHour, data.AlarmMinute, data.AlarmTimeWindow
    print data.Use_WakeUp_Song, data.WakeUp_Song_Volume,  data.WakeUp_Song
    
    root.updateScreen()
    
    if(platform.machine().startswith('arm')): view.showFullScreen()
    view.show()
    
    
    
    
    app.exec_()

    print "app closing.."
    try:
        StopMeasurement()
    except:
        pass
    WriteConfig()
    
    
    
    
    
    

###########################################################################
## Function:    MeasurementTimerLoop
## Description: This routine gets called from the timer during the recording
##              Interval: sampling_interval => 1000 ms or 1 ms if in quick mode
##              In every run it initiates a sampling series, the proccessing
##              of the data and needed actions.
## Parameters:  none
## Returns:     none
###########################################################################
def MeasurementTimerLoop():
#    print "...MeasurementTimerLoop"
    print "."
    r=GetSensorData()
#    Generate_and_show_GraphBitmap() #testing
#    root.graphGotChanged()
    #print r
    if(r==1): #new data got added to array
        GenerateDerivation(False) #Proccess new recorded data
        GenerateSleepPattern() #Recalculate sleep pattern curve (Note: This function is very time consuming)
#        ProcessAlarm() #check if alarm should get initiated

        WriteRecord() #Write recorded data into record data file

        #data.ScrollToEnd=True
        #if(data.DisplayIsOn==True): #when display is on, update graph
            #GenerateStatisticData()
#        Generate_and_show_GraphBitmap()
#        root.tellQMLToRedrawGraph()
        

        #if(records.Index==records.MaxArrayDataLen-1):
            #print "Memory is full!"
            #QtGui.QMessageBox.warning(RecordWindow, _("SleepAnalyser"), _("You reached the end of the recording capacity!\nThe recording stops now."), QtGui.QMessageBox.Ok)
            #StopMeasurement() #stop but keep alarm active!
##            SwitchModes(False) #change back to normal mode and general profile if requested

    #r=ProcessAlarm() #check if alarm should get initiated
    #if(r==1): #Start Wake Up music
        #MusicPlayer_Stop() #In case Lucid Dream music is still playing, stop it
        #MusicPlayer_Play(data.WakeUp_Song_Start_Volume, records.WakeUp_Song, records.WakeUp_Song_Duration)
    #elif(r==2): #Stop WakeUp Music
        #MusicPlayer_Stop()

    #r=ProcessLucid_Dream("") #check if lucid dream should get initiated
    #if(r==1): #Start Lucid dream music
        #duration=GetSongLength(records.Lucid_Dream_Song)
        #MusicPlayer_Play(records.Lucid_Dream_Song_Volume, records.Lucid_Dream_Song, duration)

    if(records.Index==1):#those commands are delayed from the start to here, so we get a smoother visual change of windows
        #if(data.SwitchedModes==False): #protection so it only gets switched once
            #data.SwitchedModes=True
            #print "Switch modes/profiles"
            #if(data.DeviceIsN900==True and data.QuickMode==False): #We are on a N900 and not in quick mode
                #if(data.OfflineMode==True): set_profile(False) #change to offline mode and silent profile if requested
                #if(data.SilentProfile==True):#change to silent profile
                    #data.Previous_Profile_was_silent=get_Silent()
                    #set_silent(True)
                #if(data.DimmDisplay==True): SetBrightness(data.RecordingBrightness) #dimm display brightness
                #if(data.cliStart!=""): #Expert option to run a script at recording start
                    #cmd=data.cliStart+" &"
                    #print "### Run command: ", cmd
                    #os.popen(cmd)
#        print "RECORDING..."
        pass



    #if(data.DisplayIsOn==True):#only when display is on, update clock animation
        #if(data.debug==True): print "-> o Update screen"

        #if(data.SongIsPlaying==True): RecordWindow.ui.bStopSong.setEnabled(True)
        #else: RecordWindow.ui.bStopSong.setEnabled(False)

        #RecordWindow.UpdateAnimation() #update turning clock
        #RecordWindow.UpdateScreen()


    #if(data.AlarmIsActive==True): #alarm got activated
        #if(data.StopAfterAlarm==True): #stop recording immediately
            #data.DisplayIsOn=True #Workaround, so display gets updated even display is off

            #print "Alarm got activated, stopped recording"
            #txt=_("Alarm got activated, stopped recording")
            #data.Status=txt

            #StopMeasurement() #stop but keep alarm active!

            ##change directly to view screen
            #data.LastRecord=data.RecordFilename #set to corrent file for viewwindow
            #config.set('main', 'LastRecord', str(data.LastRecord))
            #print "Show record in ViewWindow:", data.LastRecord
            #RecordWindow.hide()
            #StartWindow.ShowViewWindow()

        #else: #continue recording
            #pass

#        SwitchModes(False) #change back to normal mode and general profile if requested
        #is it really needed here?

    
    
    
    
    
    
    
    
    
###########################################################################
## Function:    InitMeasurement
## Description: It initiates the measurement and sets all needed parameters
## Parameters:  none
## Returns:     none
###########################################################################
def InitMeasurement():
    global app
    print strftime("%x %X"), "## Measurement starts..."

    data.sample_index=0
    data.last_beep=0 #used for quick mode

    #data.WakeUp_SongIsActive=False
    #data.Lucid_Dream_SongIsActive=False
    #data.SongIsPlaying=False

    #data.SwitchedModes=False
    #data.PreviousVolume=GetVolume() #store current volume as we want to restore it at end of recording

    #data.Animation=7 #start position fur turning clock pointer
    #RecordWindow.UpdateAnimation() #show turning clock

    #if(data.QuickMode==False): data.Status=_("Recording...")
    #else: data.Status=_("Recording in Test Mode...")

    records.User=data.user
    records.SWversion=data.version
    records.Sampling_interval=data.Sampling_interval
    records.Sample_smoothing=data.Sample_smoothing
    records.Value_smoothing=data.Value_smoothing
    records.Trigger=data.Trigger #save Trigger level on start
    records.statusID="OK"

    #if(data.debug==True): PrintRecordData()

    #set graph X scale and sampling interval
    if(data.QuickMode==False):
        data.interval=data.Sampling_interval #normal sampling rate
        data.Xscale=1 #set to default zoom level
    else:
        data.interval=10 #Quick mode for testing
        data.Xscale=0 #set to max zoom level

#    print "#################### USE MAXIMUM SAMPLING RATE FOR DEBUGGING!!!!!!!!!!!"
#    Note: Even it is told to run with 1 ms, in reality it will be much slower!
#    data.interval=1 #Debugging


    CreateRecordFile() #Create record data file
    r=CreateRecordFile() #Create record data file
    if(r==1): #could not open file
        return

    data.run=True #recording
    app.timer_GetData = QtCore.QTimer()
    QtCore.QObject.connect(app.timer_GetData, QtCore.SIGNAL("timeout()"), MeasurementTimerLoop)

    data.StartTime=time.localtime() #store recording start time
    data.StartTimeSec=Tuple2Seconds(data.StartTime)

    app.timer_GetData.start(data.interval) #start timer

    GetFirstEntry() #generate a dummy first entry for record data file
    WriteRecord() #Write recorded data into record data file

    Generate_and_show_GraphBitmap()

    #if(records.AlarmIsSet==True or data.QuickMode==True): #only show the trigger lower/raise buttons when alarm is set or in quick mode
        #RecordWindow.ui.bRaiseTrigger.show()
        #RecordWindow.ui.bLowerTrigger.show()
    #else:
        #RecordWindow.ui.bRaiseTrigger.hide()
        #RecordWindow.ui.bLowerTrigger.hide()

    #RecordWindow.ui.lblStatus.hide()
    #RecordWindow.ui.CanvasWidget.show()
    #RecordWindow.UpdateScreen()
    #app.processEvents() #force ui update
    
    
#    root.startRecording()

    print "=== Initialisation complete ==="
    
    
    
    
  

###########################################################################
## Function:    StopMeasurement
## Description: Stops the timer for the measurement, updates data
## Parameters:  none
## Returns:     none
###########################################################################
def StopMeasurement():
    global app
    #global MusicPlayer
    print "## Measurement stops..."

    GenerateStatisticData()
    Generate_and_show_GraphBitmap()

    #RecordWindow.Canvas.update()
    #RecordWindow.UpdateScreen()
    #app.processEvents() #force ui update

    if(data.run==True): #in recording
        if(data.debug==True): print "Stop Timer"
        app.timer_GetData.stop()
        StopRecordFile()
        #PauseSong() #stop song, in case it was playing
        #MusicPlayer_Stop() #close media player
        UpdateFileHeader() #rewrite record file (only do when recording is stopped!)

    data.run=False

    #RecordWindow.UpdateScreenStop()

    #restore profiles and modes
    #if(data.DeviceIsN900==True and data.QuickMode==False): #We are on a N900 nad are not in quick mode
        #if(data.OfflineMode==True): set_profile(True) #change to online mode
        #if(data.SilentProfile==True and data.Previous_Profile_was_silent==False): set_silent(False) #change to general profile
        #SetVolume(data.PreviousVolume) #restore previous volume
        #if(data.cliStop!=""): #Expert option to run a script at recording stop
            #cmd=data.cliStop+" &"
            #print "### Run command: ", cmd
            #os.popen(cmd)




    
    
    
    
    
###########################################################################
## Function:    Generate_and_show_GraphBitmap
## Description: Initialise, generate and show graph bitmap
## Parameters:  canvas to be drawn on
## Returns:     none
###########################################################################
def Generate_and_show_GraphBitmap():
    print "Generate_and_show_GraphBitmap"

    Xscale=data.arrXscale[data.Xscale]
    data.GraphBitMapLen=max((records.LastIndex+5)/Xscale + 2 , 649) #628 minimum graph width
    data.GraphBitMapHeight=data.Ysize+34

#    try:
#    data.GraphBitMap = QtGui.QPixmap(data.GraphBitMapLen, data.GraphBitMapHeight) #create bitmap for graph
#    data.GraphBitMap.fill(Qt.transparent)
#    
#    DrawGraph(data.GraphBitMap, QtGui) # draw graph
#    except:
#        print "Error while creating the graph bitmap"
#
#    self.Canvas.setGeometry(0, 0, data.GraphBitMapLen, data.GraphBitMapHeight)  #set size of canvas to graph
#
#    self.ui.CanvasWidget.setMinimumSize(data.GraphBitMapLen, data.GraphBitMapHeight)  #set size of canvas widget to graph
#
#    data.ScrollToEnd=True
#
#    self.ui.CanvasWidget.hide() #needed? Might force a reload
#    self.ui.CanvasWidget.show()
#    self.UpdateScreen()    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
class Graph (QDeclarativeItem):
  def __init__(self, parent = None):
    QDeclarativeItem.__init__(self, parent)    
    self.setFlag(QGraphicsItem.ItemHasNoContents, False) #We need to disable this flag to draw inside a QDeclarativeItem



  #This function creates the whole graph
  def paint(self, paint, options, widget):
    global data    
    print "paint"


    paint.setRenderHints(QPainter.Antialiasing, True);

    paint.setPen(QColor("gray"))
    paint.setBrush(QColor("black"))
    paint.setFont(QFont('Decorative', 12))

    #show graph border, background
    paint.drawRect(0,0,100,200)           

       
    
    #Formating X axis
    #Small Y lines
    for i in range(300, 2*300, 10): 
      paint.drawLine(0,i,5,i) #1 line per degree
      paint.drawLine(0,2*300-i,5,2*300-i)
      paint.drawLine(500-5,i,500,i)
      paint.drawLine(500-5,2*300-i,500,2*300-i)
    
    #Big Y lines
    for i in range(300, 2*300, 50): 
      paint.drawLine(0,i,500,i) #line per degree
      paint.drawLine(0,2*300-i,500,2*300-i)
      paint.drawText(10, i-4, str((i-300)/10))
      paint.drawText(10, 2*300-i-4, str(-1*(i-300)/10))

    #Formating Y axis
    #Small X lines
    for i in range(500, 0, -40): 
      paint.drawLine(i,300-5,i,300+5)
      
    #Big X lines
    for i in range(0, 500, 120): 
      paint.drawLine(500-i,300-20,500-i,300+20)
      if(i==0): j=i+25 
      else:j=i
      paint.drawText(500-j+5, 300+20, str(i/4))

    #Zero line
    paint.setPen(QColor("white"))
    paint.drawLine(0,300,500,300) 
    
    data.test = data.test + 1
    paint.drawText(10,  30,  str(data.test))

#    root.updateGraphWidth(300, 2*500)


    print "painting done"
    
    
    
    
###########################################################################
## Function:    SetConfig
## Description: Updates the config values
## Parameters:  self, event
## Returns:     none
###########################################################################
def SetConfig():
    #update config values
    config.set('main', 'Use_Lucid_Dream_Song', str(data.Use_Lucid_Dream_Song))
    config.set('main', 'Lucid_Dream_Song', data.Lucid_Dream_Song.encode( "utf-8" ))
    config.set('main', 'Lucid_Dream_Song_Volume', str(data.Lucid_Dream_Song_Volume))
    config.set('main', 'Lucid_Dream_Song_Delay', str(data.Lucid_Dream_Song_Delay))
    config.set('main', 'Lucid_Dream_Song_Duration', str(data.Lucid_Dream_Song_Duration))

    config.set('main', 'Use_WakeUp_Song', str(data.Use_WakeUp_Song))
    config.set('main', 'WakeUp_Song', data.WakeUp_Song.encode( "utf-8" ))
    config.set('main', 'WakeUp_Song_Volume', str(data.WakeUp_Song_Volume))
    config.set('main', 'WakeUp_Song_Duration', str(data.WakeUp_Song_Duration))

    config.set('main', 'UseAlarm', str(data.UseAlarm))
    config.set('main', 'AlarmHour', str(data.AlarmHour))
    config.set('main', 'AlarmMinute', str(data.AlarmMinute))
    config.set('main', 'AlarmTimeWindow', str(data.AlarmTimeWindow))

    config.set('main', 'UseAlternativeAlarm', str(data.UseAlternativeAlarm))

    WriteConfig() #good time to save config

    
    
    
################################################################################################    
    
class QML_to_Python_Interface(QObject):
    global data
    @Slot(bool, int, int, str, bool, int, int, int, bool, int, str)
    def StoreParameters(self, useLucidDreamSong, lucidDreamSongDelay, lucidDreamSongVolume, lucidDreamSong, useAlarm, alarmTimeHour, alarmTimeMinute, alarmTimeWindow, useWakeUpSong, wakeUpVolume, wakeUpSong):
        print "StoreParameters"
        print useLucidDreamSong, lucidDreamSongDelay, lucidDreamSongVolume, lucidDreamSong, useAlarm, alarmTimeHour, alarmTimeMinute, alarmTimeWindow, useWakeUpSong, wakeUpVolume, wakeUpSong
        
        
        data.Use_Lucid_Dream_Song = useLucidDreamSong
        data.Lucid_Dream_Song_Delay = int(lucidDreamSongDelay)
        data.Lucid_Dream_Song_Volume = int(lucidDreamSongVolume)
        data.Lucid_Dream_Song = lucidDreamSong
                
        data.UseAlarm = useAlarm
        data.AlarmHour = int(alarmTimeHour)
        data.AlarmMinute = int(alarmTimeMinute)
        data.AlarmTimeWindow = int(alarmTimeWindow)
        
        data.Use_WakeUp_Song = useWakeUpSong
        data.WakeUp_Song_Volume = int(wakeUpVolume)
        data.WakeUp_Song = wakeUpSong

        print data.Use_Lucid_Dream_Song,  data.UseAlarm,  data.Use_WakeUp_Song

        SetConfig()
    






    @Slot(result=bool)
    def StartRecording(self):
        InitRecord() #clear all old data, initialise next measurement
        
        data.QuickMode=False #run in normal mode
    
        InitMeasurement()
        
        PrintRecordData(False)
        print strftime("%x %X"), "All parameters set, start recording now.."

        print "## Open Record window..."

        return True




    @Slot(result=bool)
    def StartTestRecording(self):
        InitRecord() #clear all old data, initialise next measurement
        
        data.QuickMode=True #run in quick mode (for testing of functionality only, no data gets stored)
    
        InitMeasurement()
        
        PrintRecordData(False)
        print strftime("%x %X"), "All parameters set, start test recording now.."

        print "## Open Record window..."

        return True





    @Slot(result=bool)
    def ViewRecord(self):
        return True





    @Slot(result=bool)
    def StopRecording(self):
        print "Stop recording"
        StopMeasurement()
      
        return True
      
      
      
    
################################################################################################  
    
    

        
    
main() #start app now



    
    
    
    
    
